<?php

namespace common\logic\Distribution\Distribution;

use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\Distribution\PointSaleDistribution\PointSaleDistributionBuilder;
use common\logic\Distribution\PointSaleDistribution\PointSaleDistributionRepository;
use common\logic\Distribution\ProductDistribution\ProductDistribution;
use common\logic\Distribution\ProductDistribution\ProductDistributionBuilder;
use common\logic\Distribution\ProductDistribution\ProductDistributionRepository;
use common\logic\Order\Order\Order;
use common\logic\Order\Order\OrderRepository;
use common\logic\Producer\Producer\Producer;
use common\logic\Product\Product\Product;
use common\logic\Product\Product\ProductRepository;
use common\logic\User\UserProducer\UserProducer;

class DistributionBuilder extends BaseService implements BuilderInterface
{
    protected DistributionRepository $distributionRepository;
    protected PointSaleDistributionRepository $pointSaleDistributionRepository;
    protected PointSaleDistributionBuilder $pointSaleDistributionBuilder;
    protected ProductRepository $productRepository;
    protected ProductDistributionRepository $productDistributionRepository;
    protected ProductDistributionBuilder $productDistributionBuilder;
    protected OrderRepository $orderRepository;

    public function __construct()
    {
        $this->distributionRepository = $this->loadService(DistributionRepository::class);
        $this->pointSaleDistributionRepository = $this->loadService(PointSaleDistributionRepository::class);
        $this->pointSaleDistributionBuilder = $this->loadService(PointSaleDistributionBuilder::class);
        $this->productRepository = $this->loadService(ProductRepository::class);
        $this->productDistributionRepository = $this->loadService(ProductDistributionRepository::class);
        $this->orderRepository = $this->loadService(OrderRepository::class);
    }

    public function instanciate(Producer $producer, string $date, bool $delivery = true): Distribution
    {
        $distribution = new Distribution();

        $distribution->id_producer = $producer->id;
        $distribution->date = $date;
        $distribution->delivery = (int) $delivery;

        return $distribution;
    }

    // initDistribution
    public function create(Producer $producer, string $date, bool $delivery = true): Distribution
    {
        $distribution = $this->instanciate($producer, $date, $delivery);
        $distribution->save();

        $this->initPointSaleDistribution($distribution);
        $this->initProductDistribution($distribution);

        return $distribution;
    }

    public function createIfNotExist(Producer $producer, string $date, int $delivery = 1)
    {
        $distribution = $this->distributionRepository->getOne($producer, $date);

        if (!$distribution) {
            $distribution = $this->create($producer, $date, 1);
        }

        return $distribution;
    }

    public function initPointSaleDistribution(Distribution $distribution): void
    {
        $countPointSaleDistribution = $this->pointSaleDistributionRepository->countByDistribution($distribution);
        if (!$countPointSaleDistribution) {
            $this->pointSaleDistributionBuilder->createAll($distribution, true);
        }
    }

    public function initProductDistribution(Distribution $distribution): void
    {
        $productArray = $this->productRepository->get();
        $countProductDistribution = $this->productDistributionRepository->countByDistribution($distribution);

        if (!$countProductDistribution) {
            foreach ($productArray as $product) {
                $this->linkProduct($distribution, $product);
            }
        }
    }

    /**
     * Lie un produit à la distribution.
     */
    public function linkProduct(Distribution $distribution, Product $product): ProductDistribution
    {
        $productDistribution = $this->productDistributionBuilder->createIfNotExist($distribution, $product);
        $this->updateOrderProductPrices($distribution, $product);

        return $productDistribution;
    }

    public function updateOrderProductPrices(Distribution $distribution, Product $product): void
    {
        $ordersArray = $this->orderRepository->getByDistribution($distribution, 'AND origin != "user"');

        if ($ordersArray) {
            foreach ($ordersArray as $order) {
                foreach ($order->productOrder as $productOrder) {
                    if ($productOrder->id_product == $product->id) {
                        $productOrder->price = $product->price;

                        $user = false;
                        $userProducer = false;
                        if (isset($order->user) && $order->user) {
                            $user = $order->user;
                            $userProducer = UserProducer::searchOne([
                                'id_user' => $user->id,
                                'id_producer' => $order->distribution->id_producer
                            ]);
                        }

                        $productOrder->price = $product->getPrice([
                            'user' => $user,
                            'user_producer' => $userProducer,
                            'point_sale' => $order->pointSale,
                            'quantity' => $productOrder->quantity
                        ]);

                        $productOrder->save();
                    }
                }
            }
        }
    }
}